@@ -22,21 +22,23 @@
 """This module contains the event model with all relevant subclasses and some
 helper functions."""
 
-from collections import defaultdict
-from datetime import date, datetime, time, timedelta
-
+import datetime as dt
+import logging
 import os
+from collections import defaultdict
+
 import icalendar
 import pytz
-
-from ..utils import generate_random_uid
-from .utils import to_naive_utc, to_unix_time, invalid_timezone, delete_instance, \
-    is_aware
-from ..exceptions import FatalError
-from ..log import logger
-from ..terminal import get_color
 from click import style
 
+from ..exceptions import FatalError
+from ..terminal import get_color
+from ..utils import (cal_from_ics, delete_instance, generate_random_uid,
+                     invalid_timezone, is_aware, to_naive_utc, to_unix_time)
+from ..parse_datetime import timedelta2str
+
+logger = logging.getLogger('khal')
+
 
 class Event(object):
     """base Event class for representing a *recurring instance* of an Event
@@ -67,6 +69,7 @@ class Event(object):
         self.href = kwargs.pop('href', None)
         self.etag = kwargs.pop('etag', None)
         self.calendar = kwargs.pop('calendar', None)
+        self.color = kwargs.pop('color', None)
         self.ref = ref
 
         start = kwargs.pop('start', None)
@@ -83,7 +86,7 @@ class Event(object):
                 try:
                     self._end = self._start + self._vevents[self.ref]['DURATION'].dt
                 except KeyError:
-                    self._end = self._start + timedelta(days=1)
+                    self._end = self._start + dt.timedelta(days=1)
 
         else:
             self._end = end
@@ -96,7 +99,7 @@ class Event(object):
         :type start: icalendar.prop.vDDDTypes
         :type start: icalendar.prop.vDDDTypes
         """
-        if not isinstance(start.dt, datetime):
+        if not isinstance(start.dt, dt.datetime):
             return AllDayEvent
         if 'TZID' in start.params or start.dt.tzinfo is not None:
             return LocalizedEvent
@@ -106,9 +109,9 @@ class Event(object):
     def _get_type_from_date(cls, start):
         if hasattr(start, 'tzinfo') and start.tzinfo is not None:
             cls = LocalizedEvent
-        elif isinstance(start, datetime):
+        elif isinstance(start, dt.datetime):
             cls = FloatingEvent
-        elif isinstance(start, date):
+        elif isinstance(start, dt.date):
             cls = AllDayEvent
         return cls
 
@@ -148,18 +151,18 @@ class Event(object):
 
     @classmethod
     def fromString(cls, event_str, ref=None, **kwargs):
-        calendar_collection = icalendar.Calendar.from_ical(event_str)
+        calendar_collection = cal_from_ics(event_str)
         events = [item for item in calendar_collection.walk() if item.name == 'VEVENT']
         return cls.fromVEvents(events, ref, **kwargs)
 
     def __lt__(self, other):
         start = self.start_local
         other_start = other.start_local
-        if isinstance(start, date) and not isinstance(start, datetime):
-            start = datetime.combine(start, time.min)
+        if isinstance(start, dt.date) and not isinstance(start, dt.datetime):
+            start = dt.datetime.combine(start, dt.time.min)
 
-        if isinstance(other_start, date) and not isinstance(other_start, datetime):
-            other_start = datetime.combine(other_start, time.max)
+        if isinstance(other_start, dt.date) and not isinstance(other_start, dt.datetime):
+            other_start = dt.datetime.combine(other_start, dt.time.max)
 
         start = start.replace(tzinfo=None)
         other_start = other_start.replace(tzinfo=None)
@@ -183,8 +186,8 @@ class Event(object):
         self._vevents[self.ref].pop('DTSTART')
         self._vevents[self.ref].add('DTSTART', start)
         self._start = start
-        if not isinstance(end, datetime):
-            end = end + timedelta(days=1)
+        if not isinstance(end, dt.datetime):
+            end = end + dt.timedelta(days=1)
         self._end = end
         if 'DTEND' in self._vevents[self.ref]:
             self._vevents[self.ref].pop('DTEND')
@@ -225,7 +228,7 @@ class Event(object):
         if self.ref == 'PROTO':
             return self.start
         else:
-            return pytz.UTC.localize(datetime.utcfromtimestamp(int(self.ref)))
+            return pytz.UTC.localize(dt.datetime.utcfromtimestamp(int(self.ref)))
 
     def increment_sequence(self):
         """update the SEQUENCE number, call before saving this event"""
@@ -354,9 +357,20 @@ class Event(object):
             number = self.start_local.year - int(bday[:4])
             name = self._vevents[self.ref].get('x-fname', None)
             if int(bday[4:6]) == 2 and int(bday[6:8]) == 29:
-                return '{name}\'s {number}th birthday (29th of Feb.)'.format(name=name, number=number)
+                leap = ' (29th of Feb.)'
             else:
-                return '{name}\'s {number}th birthday'.format(name=name, number=number)
+                leap = ''
+            if (number - 1) % 10 == 0 and number != 11:
+                suffix = 'st'
+            elif (number - 2) % 10 == 0 and number != 12:
+                suffix = 'nd'
+            elif (number - 3) % 10 == 0 and number != 13:
+                suffix = 'rd'
+            else:
+                suffix = 'th'
+            return '{name}\'s {number}{suffix} birthday{leap}'.format(
+                name=name, number=number, suffix=suffix, leap=leap,
+            )
         else:
             return self._vevents[self.ref].get('SUMMARY', '')
 
@@ -368,7 +382,7 @@ class Event(object):
         """
         Decides whether we can handle a certain alarm.
         """
-        return alarm.get('ACTION') == 'DISPLAY' and isinstance(alarm.get('TRIGGER').dt, timedelta)
+        return alarm.get('ACTION') == 'DISPLAY' and isinstance(alarm.get('TRIGGER').dt, dt.timedelta)
 
     @property
     def alarms(self):
@@ -446,23 +460,27 @@ class Event(object):
         except TypeError:
             relative_to_start = relative_to_end = relative_to
 
-        if isinstance(relative_to_end, datetime):
+        if isinstance(relative_to_end, dt.datetime):
             relative_to_end = relative_to_end.date()
-        if isinstance(relative_to_start, datetime):
+        if isinstance(relative_to_start, dt.datetime):
             relative_to_start = relative_to_start.date()
 
-        if isinstance(self.start_local, datetime):
+        if isinstance(self.start_local, dt.datetime):
             start_local_datetime = self.start_local
             end_local_datetime = self.end_local
         else:
             start_local_datetime = self._locale['local_timezone'].localize(
-                datetime.combine(self.start, time.min))
+                dt.datetime.combine(self.start, dt.time.min))
             end_local_datetime = self._locale['local_timezone'].localize(
-                datetime.combine(self.end, time.min))
+                dt.datetime.combine(self.end, dt.time.min))
 
-        day_start = self._locale['local_timezone'].localize(datetime.combine(relative_to_start, time.min))
-        day_end = self._locale['local_timezone'].localize(datetime.combine(relative_to_end, time.max))
-        next_day_start = day_start + timedelta(days=1)
+        day_start = self._locale['local_timezone'].localize(
+            dt.datetime.combine(relative_to_start, dt.time.min),
+        )
+        day_end = self._locale['local_timezone'].localize(
+            dt.datetime.combine(relative_to_end, dt.time.max),
+        )
+        next_day_start = day_start + dt.timedelta(days=1)
 
         allday = isinstance(self, AllDayEvent)
 
@@ -478,6 +496,8 @@ class Event(object):
         attributes["end-date-long"] = self.end_local.strftime(self._locale['longdateformat'])
         attributes["end-time"] = self.end_local.strftime(self._locale['timeformat'])
 
+        attributes["duration"] = timedelta2str(self.duration)
+
         # should only have time attributes at this point (start/end)
         full = {}
         for attr in attributes:
@@ -707,8 +727,15 @@ class AllDayEvent(Event):
                            'which is invalid as per RFC 5545. Khal will '
                            'assume this is meant to be single-day event '
                            'on {}'.format(self.href, self.summary, self.start))
-            end += timedelta(days=1)
-        return end - timedelta(days=1)
+            end += dt.timedelta(days=1)
+        return end - dt.timedelta(days=1)
+
+    @property
+    def duration(self):
+        try:
+            return self._vevents[self.ref]['DURATION'].dt
+        except KeyError:
+            return self.end - self.start + dt.timedelta(days=1)
 
 
 def create_timezone(tz, first_date=None, last_date=None):
@@ -745,8 +772,8 @@ def create_timezone(tz, first_date=None, last_date=None):
 
     # TODO last_date = None, recurring to infinity
 
-    first_date = datetime.today() if not first_date else to_naive_utc(first_date)
-    last_date = datetime.today() if not last_date else to_naive_utc(last_date)
+    first_date = dt.datetime.today() if not first_date else to_naive_utc(first_date)
+    last_date = dt.datetime.today() if not last_date else to_naive_utc(last_date)
     timezone = icalendar.Timezone()
     timezone.add('TZID', tz)
 
@@ -763,13 +790,13 @@ def create_timezone(tz, first_date=None, last_date=None):
     first_num, last_num = 0, len(tz._utc_transition_times) - 1
     first_tt = tz._utc_transition_times[0]
     last_tt = tz._utc_transition_times[-1]
-    for num, dt in enumerate(tz._utc_transition_times):
-        if dt > first_tt and dt < first_date:
+    for num, transtime in enumerate(tz._utc_transition_times):
+        if transtime > first_tt and transtime < first_date:
             first_num = num
-            first_tt = dt
-        if dt < last_tt and dt > last_date:
+            first_tt = transtime
+        if transtime < last_tt and transtime > last_date:
             last_num = num
-            last_tt = dt
+            last_tt = transtime
 
     timezones = dict()
     for num in range(first_num, last_num + 1):
@@ -814,17 +841,9 @@ def _create_timezone_static(tz):
     timezone.add('TZID', tz)
     subcomp = icalendar.TimezoneStandard()
     subcomp.add('TZNAME', tz)
-    subcomp.add('DTSTART', datetime(1601, 1, 1))
-    subcomp.add('RDATE', datetime(1601, 1, 1))
+    subcomp.add('DTSTART', dt.datetime(1601, 1, 1))
+    subcomp.add('RDATE', dt.datetime(1601, 1, 1))
     subcomp.add('TZOFFSETTO', tz._utcoffset)
     subcomp.add('TZOFFSETFROM', tz._utcoffset)
     timezone.add_component(subcomp)
     return timezone
-
-
-class EventStandIn():
-    def __init__(self, calendar):
-        self.calendar = calendar
-        self.color = None
-        self.unicode_symbols = None
-        self.readonly = None
